<?php
/**
 * $Horde: rakim/guest/index.php,v 1.21 2004/11/22 20:04:36 chuck Exp $
 */

@define('AUTH_HANDLER', true);
@define('RAKIM_BASE', dirname(__FILE__) . '/..');
require_once RAKIM_BASE . '/lib/base.php';

if ($conf['authentication']['guest'] == 'horde' &&
    !Auth::isAuthenticated()) {
    Horde::authenticationFailureRedirect();
}

// Insert new guest into tables (guests, queues), and return guest_id.
function registerGuest($nameString, $emailString, $guestQuestion,
                       $guest_organization, $guestType, $entry_page,
                       $question_category)
{
    global $db;

    $guest_id = $db->nextId('rakim_guest_id');
    if (is_a($guest_id, 'PEAR_Error')) {
        Horde::logMessage($guest_id, __FILE__, __LINE__, PEAR_LOG_ERR);
        return array(0, 0);
    }

    $queue_id = $db->nextId('rakim_queue_id');
    if (is_a($queue_id, 'PEAR_Error')) {
        Horde::logMessage($queue_id, __FILE__, __LINE__, PEAR_LOG_ERR);
        return array(0, 0);
    }

    $login_time = time();

    $registerGuestQuery = "INSERT INTO rakim_guests (guest_id, session_id, guest_name, guest_email, guest_remote_addr, guest_user_agent, guest_login_time, guest_initial_question, guest_organization, guest_type, guest_entry_page, guest_question_category) 
    VALUES ($guest_id, " . $db->quote(session_id()) . ', ' . $db->quote($nameString) . ', ' . $db->quote($emailString) . ', ' . $db->quote(gethostbyaddr($_SERVER['REMOTE_ADDR'])) . ', ' . $db->quote($_SERVER['HTTP_USER_AGENT']) . ', ' . $db->quote($login_time) . ', ' . $db->quote($guestQuestion) . ', ' . $db->quote($guest_organization) . ', ' . $db->quote($guestType) . ', ' . $db->quote($entry_page) . ', ' . $db->quote($question_category) . ')';

    Horde::logMessage($registerGuestQuery, __FILE__, __LINE__, PEAR_LOG_DEBUG);

    $result = $db->query($registerGuestQuery);
    if (is_a($result, 'PEAR_Error')) {
        Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
        return array(0, 0, 0);
    } else {
        $setGuestQueueQuery = "INSERT INTO rakim_queues (queue_id, guest_id, queue_create_time) VALUES ($queue_id, $guest_id, " . time() . ")";
        Horde::logMessage($setGuestQueueQuery, __FILE__, __LINE__, PEAR_LOG_DEBUG);

        $result = $db->query($setGuestQueueQuery);
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
            return array(0, 0, 0);
        } else {
            return array($guest_id, $login_time, $queue_id);
        }
    }
}

// Service is not active.
if (!count($rakim->getActivePersonnel())) {
    $notification->push(_("There are no personnel available at this time. Please submit your question via our email interface."), 'horde.message');
    header('Location: ' . Horde::applicationUrl('guest/email.php', true));
    exit;
}

/**
 * Both guest_name and guest_id are already set, so they've logged in,
 * but got this page somehow.
 */
if (!empty($_SESSION['rakim']['guest_name']) && !empty($_SESSION['rakim']['guest_id'])) {
    header('Location: ' . Horde::applicationUrl('guest/rakim.php', true));
    exit;
}

// Process the form contents.
if (Util::getPost('loginTry')) {
    $authorized = true;

    // If we're doing external authentication, check it now.
    if ($conf['authentication']['guest'] == 'external' &&
        !Auth::isAuthenticated('rakim_guests')) {
        /* Get an Auth object. */
        require RAKIM_BASE . '/config/auth.php';
        $auth = &Auth::singleton($conf['rakim_guest_auth']['driver']);
        $authorized = false;

        if (isset($_POST['horde_user']) && isset($_POST['horde_pass'])) {
            /* Destroy any existing session on login and make sure to
             * use a new session ID, to avoid session fixation
             * issues. */
            Horde::getCleanSession();

            /* If authentication is successful, make sure to
             * authenticate to the 'rakim_guests' realm, not to
             * general Horde authentication. */
            if ($auth->authenticate(Util::getPost('horde_user'),
                                    array('password' => Util::getPost('horde_pass')),
                                    true, 'rakim_guests')) {
                $entry = sprintf('Login success for %s [%s] to Rakim',
                                 Auth::getAuth(), $_SERVER['REMOTE_ADDR']);
                Horde::logMessage($entry, __FILE__, __LINE__, PEAR_LOG_INFO);
                $authorized = true;
            } else {
                $reason = $auth->getLogoutErrorString();
                $entry = sprintf('FAILED LOGIN for %s [%s] to Rakim',
                                 Util::getFormData('horde_user'), $_SERVER['REMOTE_ADDR']);
                Horde::logMessage($entry, __FILE__, __LINE__, PEAR_LOG_ERR);
            }
        }
    }

    $guest_name_entered = preg_replace('|\W|', '', Util::getPost('guest_name_entered'));
    $guest_question = trim(Util::getPost('guestQuestion'));

    // Invalid guest_name_entered.
    if (!$guest_name_entered) {
        $notification->push(_("You must give a name to use this service."), 'horde.error');
    } elseif (!$guest_question) {
        $notification->push(_("Please enter an initial question."), 'horde.error');
    } elseif ($authorized) {
        $guest_email_entered = Util::getFormData('guest_email_entered');
        $guest_organization_entered = Util::getFormData('guest_organization_entered', 1);
        if (!is_numeric($guest_organization_entered)) {
            $guest_organization_entered = 1;
        }

        $_SESSION['rakim']['initial_question'] = $guest_question;

        list($newGuestID, $login_time, $queue_id) = registerGuest($guest_name_entered,
                                                                  $guest_email_entered,
                                                                  $guest_question,
                                                                  $guest_organization_entered,
                                                                  Util::getPost('guest_type'),
                                                                  Util::getPost('entry_page'),
                                                                  Util::getPost('question_category'));

        if ($newGuestID) {
            // Registration successful.
            $_SESSION['rakim']['queue_id'] = $queue_id;
            $_SESSION['rakim']['guest_id'] = $newGuestID;
            $_SESSION['rakim']['login_time'] = $login_time;
            $_SESSION['rakim']['guest_status'] = 'waiting';
            $_SESSION['rakim']['guest_name'] = $guest_name_entered;
            $_SESSION['rakim']['guest_email'] = $guest_email_entered;

            sendGuestMessage($guest_question, $_SESSION['rakim']['guest_id']);
            header('Location: ' . Horde::applicationUrl('guest/rakim.php', true));
            exit;
        }
    }
}

if ($conf['authentication']['guest'] == 'external') {
    /* Initialize the Auth credentials key. */
    require_once 'Horde/Secret.php';
    Secret::setKey('auth');
}

require RAKIM_TEMPLATES . '/common-header.inc';
require RAKIM_TEMPLATES . '/menu.inc';
require RAKIM_TEMPLATES . '/guest/index/login.inc';
require $registry->get('templates', 'horde') . '/common-footer.inc';
