<?php
/**
 * $Horde: rakim/lib/Rakim.php,v 1.39 2004/11/23 22:00:55 chuck Exp $
 */

/**
 * Insert a guest/tech message.
 */
function sendGuestMessage($messageString, $guest_id = 0, $push = 'f', $bounce = 'f',
                          $user_uid = null, $sender_name = null)
{
    global $db, $conf;

    if (is_null($user_uid)) {
        if (Rakim::isGuestURL()) {
            $user_uid = 'NULL';
            $sender_name = $_SESSION['rakim']['guest_name'];
        } else {
            $user_uid = $sender_name = Auth::getAuth();
        }
    }

    $message_id = $db->nextId('rakim_message_id');
    if (is_a($message_id, 'PEAR_Error')) {
        Horde::logMessage($message_id, __FILE__, __LINE__, PEAR_LOG_ERR);
        return false;
    }

    $sendGuestMessageQuery = 'INSERT INTO rakim_messages (message_id, message_text, user_uid, guest_id, message_sender, message_push, message_bounce, message_time)
        VALUES (' . (int)$message_id . ', ' . $db->quote($messageString) . ', ' . $db->quote($user_uid) . ', ' . (int)$guest_id . ', ' . $db->quote($sender_name) . ', ' . $db->quote($push) . ', ' . $db->quote($bounce) . ', ' . time() . ')';

    Horde::logMessage($sendGuestMessageQuery, __FILE__, __LINE__, PEAR_LOG_DEBUG);

    $result = $db->query($sendGuestMessageQuery);
    if (is_a($result, 'PEAR_Error')) {
        Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
        return false;
    }

    return $message_id;
}

function sendTechMessage($messageString, $recipient_id, $sender_id = 0, $push = 'f')
{
    global $db;

    if (!$sender_id) {
        $sender_id = Auth::getAuth();
    }

    if ($push !== 't') {
        $push = 'f';
    }

    $pmid = $db->nextId('rakim_personnel_messages_id');
    if (is_a($pmid, 'PEAR_Error')) {
        Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
        return $result;
    }

    $sendTechMessageQuery = 'INSERT INTO rakim_personnel_messages (pmessage_id, pmessage_text, user_uid_sender, user_uid_recipient, pmessage_time, pmessage_push) VALUES (' . (int)$pmid . ', ' . $db->quote($messageString) . ', ' . $db->quote($sender_id) . ', ' . $db->quote($recipient_id) . ', ' . time() . ', ' . $db->quote($push) . ')';
    Horde::logMessage($sendTechMessageQuery, __FILE__, __LINE__, PEAR_LOG_DEBUG);

    $result = $db->query($sendTechMessageQuery);
    if (is_a($result, 'PEAR_Error')) {
        Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
        return $result;
    }

    return true;
}

class Rakim {

    function setupSession()
    {
        if (!isset($_SESSION['rakim'])) {
            $_SESSION['rakim']['queue_ids'][0] = '';
            $_SESSION['rakim']['guest_ids'][0] = '';
            $_SESSION['rakim']['guest_names'][0] = '';
            $_SESSION['rakim']['guest_messages'][0] = '';
            $_SESSION['rakim']['guest_remote_addrs'][0] = '';
            $_SESSION['rakim']['available'] = true;
            $_SESSION['rakim']['techMessages'] = 0;
            $_SESSION['rakim']['techDelta'] = 0;
        }
    }

    /**
     * Take care of checking all of our various configuration options
     * for authentication.
     *
     * @return boolean  True; will exit/redirect on failure.
     */
    function checkAuthentication()
    {
        // If we're being run by a command-line script or page that
        // otherwise takes care of its own authentication, just
        // return.
        if (defined('AUTH_HANDLER')) {
            return true;
        }

        global $conf;

        // If this is a guest URL, check guest auth.
        if (Rakim::isGuestURL()) {
            if ($conf['authentication']['guest'] == 'horde' && !Auth::isAuthenticated()) {
                Horde::authenticationFailureRedirect();
            } elseif ($conf['authentication']['guest'] == 'external' && !Auth::isAuthenticated('rakim_guests')) {
                // Not authorized for the rakim_guests realm; send
                // them to the Rakim guest login page.
                header('Location: ' . Horde::applicationUrl('guest/', true));
                exit;
            }

            // Either authentication is 'none', or they've passed the
            // auth criteria.
            return true;
        }

        // This must be a regular support tech page, so check auth for
        // that.
        if ($conf['authentication']['tech'] == 'horde') {
            if (Auth::isAuthenticated()) {
                return true;
            } else {
                header('Location: ' . Horde::applicationUrl('guest/', true));
                exit;
            }
        } elseif ($conf['authentication']['tech'] == 'group') {
            if (!Auth::isAuthenticated()) {
                header('Location: ' . Horde::applicationUrl('guest/', true));
                exit;
            }

            // Check group membership here.
            require_once 'Horde/Group.php';
            $groups = &Group::singleton();
            if (!$groups->userIsInGroup(Auth::getAuth(), $conf['authentication']['tech_group'])) {
                global $notification;
                $notification->push(_("You are not authorized to use this service."), 'horde.warning');
                header('Location: ' . Horde::applicationUrl('guest/', true));
                exit;
            }
        } elseif (!Auth::isAuthenticated('rakim_techs')) {
            // Not authorized for the rakim_techs realm; send them to
            // Rakim's own login page.
            header('Location: ' . Horde::applicationUrl('login.php', true));
            exit;
        }

        // Auth criteria have been passed.
        return true;
    }

    function getTechColor($user)
    {
        static $colors, $currentColor, $assignedColors;
        if (empty($colors)) {
            $colors = array('#000099', '#990000', '#009900', '#006666', '#660066', '#993300');
            $currentColor = 0;
            $assignedColors = array();
        }

        if (empty($assignedColors[$user])) {
            $assignedColors[$user] = $colors[$currentColor];
            $currentColor++;
            if (empty($colors[$currentColor])) {
                $currentColor = 0;
            }
        }

        return $assignedColors[$user];
    }

    function getLastLogin()
    {
        static $last_login_time;

        if (empty($last_login_time)) {
            $last_login_time = 0;

            global $prefs;
            $last_login = @unserialize($prefs->getValue('last_login'));
            if (!empty($last_login['time'])) {
                $last_login_time = $last_login['time'];
            }
        }

        return $last_login_time;
    }

    function mailTranscript($emailAddress, $sid = null, $gid = null)
    {
        global $rakim;

        require_once 'Horde/MIME/Headers.php';
        require_once 'Horde/MIME/Message.php';
        require_once 'Horde/MIME/Part.php';

        $message = String::wrap(sprintf(_("This is the transcript you requested of your online support session, generated at %s on %s."), strftime('%X'), strftime('%x')), 70);
        $message .= "\n\n---------------\n\n";
        $transcript = $rakim->getTranscript($sid, $gid);
        foreach ($transcript as $row) {
            $message .= String::wrap($row[1] . ': ' . $row[0], 76) . "\n\n";
        }
        $message .= sprintf(_("\n\n---------------\n\nThis transcript was sent to %s\n"), $emailAddress);

        $mime = &new MIME_Message();
        $body = &new MIME_Part('text/plain', String::wrap($message, 76, "\n"));

        $mime->addPart($body);

        // Build the transcript headers.
        $msg_headers = &new MIME_Headers();
        $msg_headers->addMessageIdHeader();
        $msg_headers->addHeader('Date', date('r'));
        $msg_headers->addHeader('From', $emailAddress);
        $msg_headers->addHeader('To', $emailAddress);
        $msg_headers->addHeader('Subject', _("Transcript of Online Support Session"));
        $msg_headers->addMIMEHeaders($mime);

        // Send the transcript.
        return $mime->send($emailAddress, $msg_headers);
    }

    function isGuestURL()
    {
        global $registry;

        // Remove Rakim's webroot and any PATH_INFO from PHP_SELF.
        $url = str_replace($registry->get('webroot'), '', $_SERVER['PHP_SELF']);
        if (!empty($_SERVER['PATH_INFO'])) {
            $url = str_replace($_SERVER['PATH_INFO'], '', $url);
        }

        // If 'guest' is still in what's left of the URL, this is a
        // publically-accessible URL and doesn't require Horde
        // authentication.
        return strstr($url, 'guest');
    }

    function compressQueues()
    {
        $j = 0;
        $newSession = array();
        for ($i = 0; $i <= @count($_SESSION['rakim']['guest_ids']); $i++) {
            if (isset($_SESSION['rakim']['guest_ids'][$i])) {
                $newSession['guest_ids'][$j] = isset($_SESSION['rakim']['guest_ids'][$i]) ? $_SESSION['rakim']['guest_ids'][$i] : null;
                $newSession['queue_ids'][$j] = isset($_SESSION['rakim']['queue_ids'][$i]) ? $_SESSION['rakim']['queue_ids'][$i] : 0;
                $newSession['guest_names'][$j] = isset($_SESSION['rakim']['guest_names'][$i]) ? $_SESSION['rakim']['guest_names'][$i] : '';
                $newSession['guest_messages'][$j] = isset($_SESSION['rakim']['guest_messages'][$i]) ? $_SESSION['rakim']['guest_messages'][$i] : 0;
                $newSession['guest_remote_addrs'][$j] = isset($_SESSION['rakim']['guest_remote_addrs'][$i]) ? $_SESSION['rakim']['guest_remote_addrs'][$i] : null;
                $j++;
            }
        }

        $_SESSION['rakim']['queue_ids'] = $newSession['queue_ids'];
        $_SESSION['rakim']['guest_ids'] = $newSession['guest_ids'];
        $_SESSION['rakim']['guest_names'] = $newSession['guest_names'];
        $_SESSION['rakim']['guest_messages'] = $newSession['guest_messages'];
        $_SESSION['rakim']['guest_remote_addrs'] = $newSession['guest_remote_addrs'];
    }

    /**
     * Build Rakim's list of menu items.
     *
     * @access public
     */
    function getMenu($returnType = 'object')
    {
        global $conf, $registry;

        require_once 'Horde/Menu.php';
        $menu = &new Menu();

        if (Rakim::isGuestURL()) {
            $menu->add(Horde::applicationUrl('guest/index.php'), _("Live Support"), 'rakim.png', null, '_top');
            $menu->add(Horde::applicationUrl('guest/email.php'), _("Email Question"), 'mail.png', $registry->getImageDir('horde'), '_top');

            /* Put the close/logout button at the very end of the
             * menu. */
            if (strpos($_SERVER['PHP_SELF'], 'messages.php') !== false) {
                $id = $menu->add(Horde::applicationUrl('guest/logout.php'), _("Log out"), 'logout.png', $registry->getImageDir('horde'), '_top', 'return window.confirm(\'' . addslashes(_("Are you sure you want to leave the support system?")) . '\');');
            } else {
                $id = $menu->add('', _("Close Window"), 'close.png', $registry->getImageDir('horde'), null, 'window.close();');
            }
            $menu->setPosition($id, HORDE_MENU_POS_LAST);
        } else {
            $menu->add(Horde::applicationUrl('home.php'), _("Home"), 'rakim.png', null, null, null, basename($_SERVER['PHP_SELF']) == 'index.php' ? 'current' : null);
            $menu->add(Horde::applicationUrl('emailList.php'), _("Emails"), 'mail.png', $registry->getImageDir('horde'));

            /* Administration. */
            if (Auth::isAdmin('rakim:admin')) {
                $menu->add(Horde::applicationUrl('reports.php'), _("Reports"), 'reports.png');
                $menu->add(Horde::applicationUrl('admin.php'), _("Admin"), 'administration.png', $registry->getImageDir('horde'));
            }

            /* Put the logout button at the very end of the menu. */
            if ($conf['authentication']['tech'] == 'external') {
                $id = $menu->add(Auth::addLogoutParameters(Horde::applicationUrl('login.php'), AUTH_REASON_LOGOUT), _("Logout"), 'logout.png', $registry->getImageDir('horde'));
                $menu->setPosition($id, HORDE_MENU_POS_LAST);
            }
        }

        if ($returnType == 'object') {
            return $menu;
        } else {
            return $menu->render();
        }
    }

}

class Rakim_Driver {

    /**
     * Attempts to return a concrete Rakim_Driver instance based on
     * $driver.
     *
     * @param string    $driver     The type of concrete Rakim_Driver subclass
     *                              to return.  The is based on the storage
     *                              driver ($driver).  The code is dynamically
     *                              included.
     * @param array     $params     (optional) A hash containing any additional
     *                              configuration or connection parameters a
     *                              subclass might need.
     *
     * @return mixed    The newly created concrete Rakim_Driver instance, or
     *                  false on an error.
     */
    function &factory($driver = null, $params = null)
    {
        if (is_null($driver)) {
            $driver = $GLOBALS['conf']['backend']['driver'];
        }

        $driver = basename($driver);

        if (is_null($params)) {
            $params = Horde::getDriverConfig('backend', $driver);
        }

        require_once dirname(__FILE__) . '/Driver/' . $driver . '.php';
        $class = 'Rakim_Driver_' . $driver;
        if (class_exists($class)) {
            return $ret = &new $class($params);
        } else {
            return false;
        }
    }

    /**
     * Attempts to return a reference to a concrete Rakim_Driver
     * instance based on $driver. It will only create a new instance
     * if no Rakim_Driver instance with the same parameters currently
     * exists.
     *
     * This method must be invoked as: $var = &Rakim_Driver::singleton()
     *
     * @param string    $driver     The type of concrete Rakim_Driver subclass
     *                              to return.  The is based on the storage
     *                              driver ($driver).  The code is dynamically
     *                              included.
     * @param array     $params     (optional) A hash containing any additional
     *                              configuration or connection parameters a
     *                              subclass might need.
     *
     * @return mixed    The created concrete Rakim_Driver instance, or false
     *                  on error.
     */
    function &singleton($driver = null, $params = null)
    {
        static $instances;

        if (is_null($driver)) {
            $driver = $GLOBALS['conf']['backend']['driver'];
        }

        if (is_null($params)) {
            $params = Horde::getDriverConfig('backend', $driver);
        }

        if (!isset($instances)) {
            $instances = array();
        }

        $signature = serialize(array($driver, $params));
        if (!isset($instances[$signature])) {
            $instances[$signature] = &Rakim_Driver::factory($driver, $params);
        }

        return $instances[$signature];
    }

}
