<?php
/**
 * $Horde: rakim/monitor.php,v 1.26 2004/10/22 19:39:44 chuck Exp $
 */

@ignore_user_abort(true);
define('RAKIM_BASE', dirname(__FILE__));
require_once RAKIM_BASE . '/lib/base.php';

header('Expires: -1');
header('Pragma: no-cache');
header('Cache-Control: no-store, no-cache, must-revalidate, post-check=0, pre-check=0');

function countQueue()
{
    global $db;

    $countQueueQuery = "SELECT count(*) AS queueCount FROM rakim_queues q, rakim_guests g
        WHERE (q.user_uid IS NULL OR q.user_uid = " . $db->quote(Auth::getAuth()) . ")
            AND g.guest_id = q.guest_id
            AND g.guest_logoff_time IS NULL
            AND (q.queue_status = 'waiting' OR q.queue_status = 'referral' OR q.queue_status = 'invite')";

    Horde::logMessage($countQueueQuery, __FILE__, __LINE__, PEAR_LOG_DEBUG);

    $queueCount = $db->getOne($countQueueQuery);
    if (is_a($queueCount, 'PEAR_Error')) {
        Horde::logMessage($queueCount, __FILE__, __LINE__, PEAR_LOG_ERR);
        return 0;
    } else {
        return $queueCount;
    }
}

/**
 * Make sure we have all of the tabs/queues displayed that we should.
 */
function updateQueues()
{
    global $db;

    $queueQuery = "SELECT q.queue_id AS queue_id, q.guest_id AS guest_id, g.guest_name AS guest_name, g.guest_remote_addr AS remote_addr, g.guest_organization AS guest_organization, g.guest_entry_page as entry_page, g.guest_user_agent AS user_agent
        FROM rakim_guests g, rakim_queues q
        WHERE g.guest_id = q.guest_id
        and q.user_uid = " . $db->quote(Auth::getAuth()) . " AND queue_status = 'active'";

    Horde::logMessage($queueQuery, __FILE__, __LINE__, PEAR_LOG_DEBUG);

    $queues = $db->getAll($queueQuery, null, DB_FETCHMODE_ASSOC);
    if (is_a($queues, 'PEAR_Error')) {
        Horde::logMessage($queues, __FILE__, __LINE__, PEAR_LOG_ERR);
        return false;
    }

    if (!@is_array($_SESSION['rakim']['queue_ids'])) {
        $_SESSION['rakim']['queue_ids'][0] = '';
    }
    $changed = false;
    foreach ($queues as $queue) {
        if (!in_array($queue['queue_id'], $_SESSION['rakim']['queue_ids'])) {
            $changed = true;

            // Some of these arrays won't be set for the 0 (support)
            // queue, so calculate the common index for all of them.
            $guestNumber = count($_SESSION['rakim']['queue_ids']);

            $_SESSION['rakim']['queue_ids'][$guestNumber] = $queue['queue_id'];
            $_SESSION['rakim']['guest_ids'][$guestNumber] = $queue['guest_id'];
            $_SESSION['rakim']['guest_names'][$guestNumber] = $queue['guest_name'];
            $_SESSION['rakim']['guest_remote_addrs'][$guestNumber] = $queue['remote_addr'];
            $_SESSION['rakim']['guest_organizations'][$guestNumber] = $queue['guest_organization'];
            $_SESSION['rakim']['entry_pages'][$guestNumber] = $queue['entry_page'];
            $_SESSION['rakim']['user_agents'][$guestNumber] = $queue['user_agent'];
        }
    }

    if (empty($_SESSION['rakim']['queue_ids'][$_SESSION['rakim']['guestNumber']])) {
        $_SESSION['rakim']['guestNumber'] = 0;
    }

    return $changed;
}

function countTechMessages()
{
    global $db;

    $countTechMessagesQuery = "SELECT count(*) FROM rakim_personnel_messages
        WHERE (user_uid_sender = " . $db->quote(Auth::getAuth()) . " OR user_uid_recipient = '0' OR user_uid_recipient = " . $db->quote(Auth::getAuth()) . ")
        AND pmessage_time >= " . Rakim::getLastLogin();

    Horde::logMessage($countTechMessagesQuery, __FILE__, __LINE__, PEAR_LOG_DEBUG);

    $messagesCount = $db->getOne($countTechMessagesQuery); 
    if (is_a($messagesCount, 'PEAR_Error')) {
        Horde::logMessage($messagesCount, __FILE__, __LINE__, PEAR_LOG_ERR);
        return 0;
    } else {
        return $messagesCount;
    }
}

function countGuestMessages($guestNumber)
{
    global $db;

    $countGuestMessagesQuery = "SELECT count(*) FROM rakim_messages 
    WHERE guest_id=" . $_SESSION['rakim']['guest_ids'][$guestNumber] . " AND message_time >= '" . Rakim::getLastLogin() . "'";

    Horde::logMessage($countGuestMessagesQuery, __FILE__, __LINE__, PEAR_LOG_DEBUG);

    $messagesCount = $db->getOne($countGuestMessagesQuery); 
    if (is_a($messagesCount, 'PEAR_Error')) {
        Horde::logMessage($messagesCount, __FILE__, __LINE__, PEAR_LOG_ERR);
        return 0;
    } else {
        return $messagesCount;
    }
}

require RAKIM_TEMPLATES . '/common-header.inc';

$rakim->updateLastActive();

$queueCount = countQueue();
if (!isset($_SESSION['rakim']['prevQueue']) ||
    $queueCount != $_SESSION['rakim']['prevQueue']) {
    $_SESSION['rakim']['prevQueue'] = $queueCount;
    ?>
    <script language="JavaScript" type="text/javascript">
    window.setTimeout("top.rakim_queue.location.href = '<?php echo Horde::applicationUrl(Util::nocacheUrl('queue.php'), true) ?>'", 0);
    </script>
    <?php
}

/**
 * Every once in a while, tell tabs.php to re-check active queues, in
 * case one was lost accidentally.
 */
if (empty($_SESSION['rakim']['tabReload'])) {
    $_SESSION['rakim']['tabReload'] = time();
}
if (time() - $_SESSION['rakim']['tabReload'] > 120) {
    $_SESSION['rakim']['tabReload'] = time();
    if (updateQueues()) {
        Rakim::compressQueues();
    ?>
    <script language="JavaScript" type="text/javascript">
    window.setTimeout("top.rakim_tabs.location.href = '<?php echo Horde::applicationUrl('tabs.php') ?>';", 0);
    </script>
    <?php
    }
}

if (!@count($_SESSION['rakim']['guest_ids'])) {
    $_SESSION['rakim']['guest_ids'] = array(0);
}
foreach ($_SESSION['rakim']['guest_ids'] as $i => $guest_id) {
    if (empty($_SESSION['rakim']['guest_messages'][$i])) {
        $_SESSION['rakim']['guest_messages'][$i] = 0;
    }

    if ($i == 0) {
        $messagesCount = countTechMessages();
        if ($messagesCount != $_SESSION['rakim']['guest_messages'][$i]) {
            $_SESSION['rakim']['guest_messages'][$i] = $messagesCount;
            if (empty($_SESSION['rakim']['guestNumber'])) {
                $_SESSION['rakim']['techDelta'] = $messagesCount - $_SESSION['rakim']['guest_messages'][$i];
                ?>
                <script language="JavaScript" type="text/javascript">
                window.setTimeout("top.rakim_messages.location.href = '<?php echo Horde::applicationUrl('messages.php') ?>'", 0);
                </script>
                <?php
            } else {
                ?>
                <script language="JavaScript" type="text/javascript">
                tab = top.rakim_tabs.document.getElementById(<?php echo $i ?>);
                if (tab) {
                    tab.className = 'notifyTab';
                }
                </script>
                <?php
            }
        }
    } else {
        $messagesCount = countGuestMessages($i);
        if ($messagesCount != $_SESSION['rakim']['guest_messages'][$i]) {
            $_SESSION['rakim']['guest_messages'][$i] = $messagesCount;
            if ($_SESSION['rakim']['guestNumber'] == $i) {
                $_SESSION['rakim']['guestDelta'][$i] = $messagesCount - $_SESSION['rakim']['guest_messages'][$i];
                ?>
                <script language="JavaScript" type="text/javascript">
                window.setTimeout("top.rakim_messages.location.href = '<?php echo Horde::applicationUrl('messages.php') ?>'", 0);
                </script>
                <?php
            } else {
                ?>
                <script language="JavaScript" type="text/javascript">
                tab = top.rakim_tabs.document.getElementById(<?php echo $i ?>);
                if (tab) {
                    tab.className = 'notifyTab';
                }
                </script>
                <?php
            }
        }
    }
}

?>
<script language="JavaScript" type="text/javascript">
window.setTimeout("window.location.reload(true);", 5000);
</script>
<?php

require $registry->get('templates', 'horde') . '/common-footer.inc';
