#!/usr/local/bin/php -f
<?php
/**
 * $Horde: rakim/scripts/archive.php,v 1.18 2004/04/07 14:43:43 chuck Exp $
 */

@define('AUTH_HANDLER', true);
@define('HORDE_BASE', dirname(__FILE__) . '/../..');
@define('RAKIM_BASE', dirname(__FILE__) . '/..');

// Do CLI checks and environment setup first.
require_once HORDE_BASE . '/lib/core.php';
require_once 'Horde/CLI.php';

// Make sure no one runs this from the web.
if (!Horde_CLI::runningFromCLI()) {
    exit("Must be run from the command line\n");
}

// Load the CLI environment - make sure there's no time limit, init
// some variables, etc.
Horde_CLI::init();

// Now go ahead and load Rakim.
require_once RAKIM_BASE . '/lib/base.php';
require_once 'Horde/Identity.php';

// These are the timestamps we will be using to determine what data to
// archive/delete; anything older than these timestamps will be
// archived.
$chat_timestamp = time() - ($conf['stats']['archive_chat'] * 86400);
$email_timestamp = time() - ($conf['stats']['archive_email'] * 86400);


// First pull data older than $chat_timestamp and generate the report
// tables for it.
$query = 'SELECT q.queue_id, g.guest_id, g.guest_type, g.guest_user_agent, g.guest_question_category, q.queue_question_types, q.queue_resolution, q.user_uid,' .
    ' o.organization_name, g.guest_login_time, g.guest_logoff_time,' .
    ' q.queue_create_time, q.queue_active_time, q.queue_exit_time' .
    ' FROM rakim_queues q LEFT JOIN rakim_guests g on q.guest_id = g.guest_id' .
    '                     LEFT JOIN rakim_organizations o on g.guest_organization = o.organization_id' .
    ' WHERE g.guest_logoff_time < ' . (int)$chat_timestamp . ' AND q.queue_status = \'exit\'';
Horde::logMessage($query, __FILE__, __LINE__, PEAR_LOG_DEBUG);

$data = $db->getAll($query, DB_FETCHMODE_ASSOC);
dbCheck($data);

// We're going to store the queue IDs we archive for later use in
// cleaning up/deleting data.
$guestIds = array();

// Run inserts into the archive table.
$i = 0;
foreach ($data as $row) {
    $guestIds[] = $row['guest_id'];

    // Calculate duration.
    $row['duration'] = $row['guest_logoff_time'] - $row['guest_login_time'];

    // Calculate a bunch of static date fields, very useful for
    // report/graph generation.
    if (!empty($row['queue_active_time'])) {
        list($row['year'], $row['month'], $row['week'], $row['day'], $row['dow'], $row['doy'], $row['hour']) = explode(':', date('Y:n:W:j:w:z:G', $row['queue_active_time']));
    } else {
        list($row['year'], $row['month'], $row['week'], $row['day'], $row['dow'], $row['doy'], $row['hour']) = explode(':', date('Y:n:W:j:w:z:G', $row['queue_create_time']));
    }

    // Get the support person's name.
    if (!empty($row['user_uid'])) {
        $identity = &Identity::singleton('none', $row['user_uid']);
        $row['user_name'] = $identity->getValue('fullname');
    } else {
        $row['user_name'] = '';
    }

    // Make sure the organization isn't null - could happen if no
    // organizations have been created.
    if (empty($row['organization_name'])) {
        $row['organization_name'] = 'default';
    }

    // Get a new primary key.
    $id = $db->nextId('rakim_archived_sessions');
    dbCheck($id);

    $query = 'INSERT INTO rakim_archived_sessions (session_id, session_guest_id, session_guest_type, session_guest_user_agent, session_question_category, session_guest_login_time, session_guest_logoff_time, session_question_types, session_resolution, session_user_uid, session_user_name, session_organization,' .
        ' session_create_time, session_active_time, session_exit_time, session_year, session_month, session_week, session_day, session_dow, session_doy, session_hour)' .
        ' VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)';
    $query = sprintf($query,
                     (int)$id,
                     (int)$row['guest_id'],
                     $db->quote($row['guest_type']),
                     $db->quote($row['guest_user_agent']),
                     $db->quote($row['guest_question_category']),
                     (int)$row['guest_login_time'],
                     (int)$row['guest_logoff_time'],
                     $db->quote($row['queue_question_types']),
                     $db->quote($row['queue_resolution']),
                     $db->quote($row['user_uid']),
                     ($row['user_name'] ? $db->quote($row['user_name']) : "''"),
                     $db->quote($row['organization_name']),
                     (int)$row['queue_create_time'],
                     (int)$row['queue_active_time'],
                     (int)$row['queue_exit_time'],
                     (int)$row['year'],
                     (int)$row['month'],
                     (int)$row['week'],
                     (int)$row['day'],
                     (int)$row['dow'],
                     (int)$row['doy'],
                     (int)$row['hour']);

    dbCheck($db->query($query));
    $i++;
}

if ($i) {
    debug("Successfully inserted $i rows to the archived sessions table.\n\n");
} else {
    debug("No sessions currently meet the archiving criteria.\n\n");
}


// Now archive transcripts. We already have $guestIds to use to get
// all matching messages here:
if (count($guestIds)) {
    $messages = $db->query('SELECT message_id, message_text, user_uid, guest_id, message_time FROM rakim_messages' .
                           ' WHERE guest_id IN (' . implode(',', $guestIds) . ')');
    dbCheck($messages);

    $i = 0;
    while ($message = $messages->fetchRow(DB_FETCHMODE_ASSOC)) {
        $tid = $db->nextId('rakim_archived_transcripts');
        dbCheck($tid);

        $query = sprintf('INSERT INTO rakim_archived_transcripts (transcript_id, guest_id, transcript_sender, transcript_time, transcript_text)' .
                         ' VALUES (%s, %s, %s, %s, %s)',
                         (int)$tid,
                         (int)$message['guest_id'],
                         $db->quote(empty($message['user_uid']) ? 'guest' : 'support'),
                         (int)$message['message_time'],
                         $db->quote($message['message_text']));

        dbCheck($db->query($query));
        $i++;
    }

    debug("Successfully inserted $i rows to the archived transcripts table.\n\n");


    // Now delete the archived transcripts, messages, and queues to keep
    // those tables nice and pruned.
    dbCheck($db->query('DELETE FROM rakim_messages WHERE guest_id IN (' . implode(', ', $guestIds) . ')'));
    dbCheck($db->query('DELETE FROM rakim_queues WHERE guest_id IN (' . implode(', ', $guestIds) . ')'));
    dbCheck($db->query('DELETE FROM rakim_guests WHERE guest_id IN (' . implode(', ', $guestIds) . ')'));

    debug("Successfully deleted archived chat data from the live tables.\n\n");
}

// Also prune tech-to-tech messages.
dbCheck($db->query('DELETE FROM rakim_personnel_messages WHERE pmessage_time < ' . (int)$chat_timestamp));
debug("Successfully deleted old tech-to-tech messages.\n\n");


// Now archive emails that have been closed since $email_timestamp or
// before.
$emails = $db->getAll('SELECT e.*, o.organization_name FROM rakim_emails e LEFT JOIN rakim_organizations o on e.guest_organization = o.organization_id WHERE e.email_time_closed < ' . (int)$email_timestamp . ' AND e.email_status = \'closed\'', DB_FETCHMODE_ASSOC);
dbCheck($emails);

// We're going to store the emails IDs we archive for later use in
// cleaning up/deleting data.
$emailIds = array();

// Run inserts into the archive table.
$i = 0;
foreach ($emails as $row) {
    // Calculate a bunch of static date fields, very useful for
    // report/graph generation.
    list($row['year'], $row['month'], $row['week'], $row['day'], $row['dow'], $row['doy'], $row['hour']) = explode(':', date('Y:n:W:j:w:z:G', $row['email_time_asked']));

    // Make sure the organization isn't null - could happen if no
    // organizations have been created.
    if (empty($row['organization_name'])) {
        $row['organization_name'] = 'default';
    }

    // Get a new primary key.
    $id = $db->nextId('rakim_archived_emails');
    dbCheck($id);

    $emailIds[$id] = $row['email_id'];

    $query = 'INSERT INTO rakim_archived_emails (email_id, email_text, email_time_asked, email_time_claimed, email_time_answered, email_time_closed,' .
        ' email_year, email_month, email_week, email_day, email_dow, email_doy, email_hour,' .
        ' guest_id, guest_organization, guest_question_category, guest_type, guest_user_agent)' .
        ' VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)';
    $query = sprintf($query,
                     (int)$id,
                     $db->quote($row['email_text']),
                     (int)$row['email_time_asked'],
                     (int)$row['email_time_claimed'],
                     (int)$row['email_time_answered'],
                     (int)$row['email_time_closed'],
                     (int)$row['year'],
                     (int)$row['month'],
                     (int)$row['week'],
                     (int)$row['day'],
                     (int)$row['dow'],
                     (int)$row['doy'],
                     (int)$row['hour'],
                     (int)$row['guest_id'],
                     $db->quote($row['organization_name']),
                     $db->quote($row['guest_question_category']),
                     $db->quote($row['guest_type']),
                     $db->quote($row['guest_user_agent']));

    dbCheck($db->query($query));
    $i++;
}

if ($i) {
    debug("Successfully inserted $i rows to the archived emails table.\n\n");
} else {
    debug("No emails currently meet the archiving criteria.\n\n");
}

if (count($emailIds)) {
    // Archive any responses for the archived emails.
    $responses = $db->getAll('SELECT * from rakim_email_responses WHERE email_id IN (' . implode(',', $emailIds) . ')', DB_FETCHMODE_ASSOC);
    dbCheck($responses);

    $emailMap = array_flip($emailIds);
    $i = 0;
    foreach ($responses as $row) {
        $rid = $db->nextId('rakim_archived_email_responses');
        dbCheck($rid);

        $query = sprintf('INSERT INTO rakim_archived_email_responses (response_id, email_id, response_time, response_text) VALUES (%s, %s, %s, %s)',
                         $rid,
                         $emailMap[$row['email_id']],
                         (int)$row['response_time'],
                         $db->quote($row['response_text']));

        dbCheck($db->query($query));
        $i++;
    }

    if ($i) {
        debug("Successfully inserted $i rows to the archived email responses table.\n\n");
    } else {
        debug("No email responses currently meet the archiving criteria.\n\n");
    }

    // Now delete the archived emails and responses to keep those
    // tables nice and pruned.
    dbCheck($db->query('DELETE FROM rakim_email_responses WHERE email_id IN (' . implode(', ', $emailIds) . ')'));
    dbCheck($db->query('DELETE FROM rakim_emails WHERE email_id IN (' . implode(', ', $emailIds) . ')'));

    debug("Successfully deleted archived email data from the live tables.\n\n");
}


/**
 * Helper function for error checking and exiting if necessary.
 *
 * @param object PEAR_Error $result  The result to check.
 */
function dbCheck($result)
{
    if (is_a($result, 'PEAR_Error')) {
        Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
        echo "An error occurred:\n\n";
        echo $result->getMessage();
        echo "\n";
        echo $result->getUserInfo();
        echo "\n";
        exit;
    }
}

/**
 * Only send output if debugging is turned on. Avoids sending cron
 * mail when there are no errors.
 *
 * @param string $output  The string to echo.
 */
function debug($output)
{
    global $DEBUG;

    if (!empty($DEBUG)) {
        echo $output;
    }
}
